#include <oxstd.h>

#include "bprobit.h"


/*-------------------------- Probit : Database -----------------------------*/
Probit::Probit()
{
    this->Database();           // intialize base class
    m_iModel = 0;               // 1 when estimated
    m_fPrint = TRUE;
    m_iResult = -1;

    m_mPar = 0;

    print("Probit class example 4, object created on ",
        date(), ".\n\n");
}

Probit::fProbit(const vP, const adFunc, const avScore,
    const amHessian)
{
    decl prob = probn(m_mX * vP);
    decl tail = 1 - prob;

    adFunc[0] = double(
        meanc(m_mY .* log(prob) + (1-m_mY) .* log(tail)));

    if (avScore)
    {
        decl weight = (m_mY - prob) .* densn(m_mX * vP)
            ./ (prob .* tail);
        avScore[0] = meanc(weight .* m_mX)';
    }

return 1;                           // 1 indicates success
}

Probit::InitData()
{
    decl  cp, vp, mh;

    m_iT1est = m_iT1sel;  m_iT2est = m_iT2sel;

    m_mY = GetGroupLag(Y_VAR, 0, 0);
    m_cY = columns(m_mY);

    if (m_cY != 1)
    {   eprint("Only univariate models allowed\n");
        return FALSE;
    }
    m_mX = GetGroup(X_VAR);
    m_cX = columns(m_mX);
    if (!m_cX)
    {   eprint("Need some regressors\n");
        return FALSE;
    }
    m_cT = m_iT2est - m_iT1est + 1;
    if (m_cT <= 2)
    {   eprint("Only ", m_cT, " observations. This is not enough.\n");
        return FALSE;
    }

    return TRUE;
}

Probit::Estimate(const vStart)
{
    decl  cp;

    m_iResult = MAX_FUNC_FAIL;        // code from MaxBFGS
    if (!InitData())
    {   print("Failed to load data\n");
        m_mPar = zeros(1, m_cX);
        return m_iResult;
    }
    if (sizer(vStart))
        m_mPar = vStart;
    if (m_fPrint)
        print("starting values: ", m_mPar);

    cp = sizer(m_mPar);

    m_iResult = MaxBFGS(fProbit, &m_mPar, &m_dLoglik, 0, FALSE);
    m_dLoglik *= m_cT;

    if (m_fPrint)
        Output();
    return m_iResult;
}

Probit::IsConverged()
{
    return (m_iResult == MAX_CONV || m_iResult == MAX_WEAK_CONV);
}
Probit::GetParameters()
{
    return m_mPar;
}
Probit::GetStandardErrors()
{
    decl mhess;
    if (Num2Derivative(fProbit, m_mPar, &mhess))
    {
        mhess = -invert(mhess) / m_cT;
        return sqrt(diagonal(mhess)');
    }
    return 0;
}


Probit::Output()
{
    print("\n", MaxConvergenceMsg(m_iResult),
        " using analytical derivatives",
        "\nLog-likelihood = ", m_dLoglik, "; parameters:", m_mPar);

                  // if converged: compute standard errors
    if (IsConverged())
         print("standard errors:", GetStandardErrors());
}
Probit::SetPrint(const fPrint)
{
    m_fPrint = fPrint;
}
